/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.java.sip.communicator.plugin.addrbook;

/**
 * Defines the interface for a callback function which is called by the native
 * counterpart of the support for the OS-specific Address Book with a pointer as
 * its argument.
 *
 * @author Lyubomir Marinov
 */
public interface PtrCallback
{
    /**
     * Notifies this <tt>PtrCallback</tt> about a specific pointer.
     *
     * @param ptr the pointer to notify this <tt>PtrCallback</tt> about
     * @return <tt>true</tt> if this <tt>PtrCallback</tt> is to continue being
     * called; otherwise, <tt>false</tt>
     */
    boolean callback(long ptr);
}
