/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.java.sip.communicator.plugin.desktoputil;

import java.awt.*;

/**
 * Through the <tt>AntialiasingManager</tt> the developer could activate the
 * antialiasing mechanism when painting. The method that do the job is
 * the <code>activateAntialiasing</code> method. It takes a <tt>Graphics</tt>
 * object and activates the antialiasing for it.
 *
 * @author Yana Stamcheva
 */
public class AntialiasingManager
{

    /**
     * Activates the antialiasing mechanism for the given <tt>Graphics</tt>
     * object.
     * @param g The <tt>Graphics</tt> object.
     */
    public static void activateAntialiasing(Graphics g)
    {
        ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                        RenderingHints.VALUE_ANTIALIAS_ON);
    }
}
