/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _MSOUTLOOKDLL_H
#define _MSOUTLOOKDLL_H
#include <windows.h>
#include <jni.h>

#ifdef __cplusplus
extern "C" {
#endif

JNIEXPORT BOOL APIENTRY DllMain(
    HINSTANCE hDLL,
    DWORD dwReason,
    LPVOID lpReserved);

#ifdef __cplusplus
}
#endif

#endif //_MSOUTLOOKDLL_H
