/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "run.h"

#ifndef RT_MANIFEST
#define RT_MANIFEST 24
#endif /* #ifndef RT_MANIFEST */
1 RT_MANIFEST "asInvoker.exe.manifest"

SCLogoIcon ICON "sc-logo.ico"

STRINGTABLE
BEGIN
    IDS_CRASHANDRELAUNCH "Ouch, it seems %1 has crashed!\012\012Relaunch it?"
    IDS_JAVANOTFOUND "A Java Runtime Environment (JRE) or Java Development Kit (JDK) must be available in order to run %1."
END

1 VERSIONINFO
    FILEVERSION PRODUCTBUILDVERSION_COMMA
    FILETYPE VFT_APP
    PRODUCTVERSION 0,0,0,0
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904E4"
        BEGIN
            VALUE "Comments", ""
            VALUE "CompanyName", COMPANYNAME
            VALUE "FileDescription", PRODUCTNAME
            VALUE "FileVersion", PRODUCTBUILDVERSION
            VALUE "InternalName", TARGET_BASENAME
            VALUE "OriginalFilename", TARGET_BASENAME_EXE
            VALUE "ProductName", PRODUCTNAME
            VALUE "ProductVersion", PRODUCTBUILDVERSION
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0409, 1252
    END
END
