/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class net_java_sip_communicator_impl_netaddr_Win32LocalhostRetriever */

#ifndef _Included_net_java_sip_communicator_impl_netaddr_Win32LocalhostRetriever
#define _Included_net_java_sip_communicator_impl_netaddr_Win32LocalhostRetriever
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     net_java_sip_communicator_impl_netaddr_Win32LocalhostRetriever
 * Method:    getSourceForDestination
 * Signature: ([B)[B
 */
JNIEXPORT jbyteArray JNICALL Java_net_java_sip_communicator_impl_netaddr_Win32LocalhostRetriever_getSourceForDestination
  (JNIEnv *, jclass, jbyteArray);

#ifdef __cplusplus
}
#endif
#endif
