/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _JMSOFFICECOMM_MESSENGERSERVICES_H_
#define _JMSOFFICECOMM_MESSENGERSERVICES_H_

#include "DispatchImpl.h"
#include <msgrua.h>

class MessengerServices
    : public DispatchImpl<IMessengerServices, IID_IMessengerServices>
{
public:
    MessengerServices(IMessenger *messenger);

    // IMessengerServices
    STDMETHODIMP get_PrimaryService(IDispatch **ppService);
    STDMETHODIMP get_Count(long *pcServices);
    STDMETHODIMP Item(long Index, IDispatch **ppService);
    STDMETHODIMP get__NewEnum(IUnknown **ppUnknown);

protected:
    virtual ~MessengerServices();

private:
    IMessenger *_messenger;
    IWeakReference *_primaryService;
};

#endif /* _JMSOFFICECOMM_MESSENGERSERVICES_H_ */ 
