/* Tiger Mac Help task_style.css v2.0f*/
/* <html><head><meta name="robots" content="noindex"></head></html> */

body  {
	margin: 2px 12px 12px
	}

h1            {
	font-size: 11pt;
	font-family: "Lucida Grande", Arial, sans-serif;
	margin-bottom: 0
	}


h2               {
	font-size: 9pt;
	font-family: "Lucida Grande", Arial, sans-serif;
	margin-top: 0;
	margin-bottom: -10px
	}

h3   {
	font-size: 9pt;
	font-family: "Lucida Grande", Arial, sans-serif;
	font-weight: bold;
	margin-top: -4px;
	margin-bottom: -4px
	}

p	           {
	font-size: 9pt;
	font-family: "Lucida Grande", Arial, sans-serif;
	line-height: 12pt;
	text-decoration: none
	}


ol                     {
	font-size: 9pt;
	font-family: "Lucida Grande", Arial, sans-serif;
	line-height: 12pt;
	list-style-position: outside;
	margin-top: 12px;
	margin-bottom: 12px;
	margin-left: -18px;
	padding-left: 40px
	}


ol li         {
	margin-top: 6px;
	margin-bottom: 6px
	}


ol p        {
	margin-top: 6px;
	margin-bottom: 6px
	}


ul                     {
	font-size: 9pt;
	font-family: "Lucida Grande", Arial, sans-serif;
	line-height: 12pt;
	list-style-type: circle;
	list-style-position: outside;
	margin-top: 1px;
	margin-bottom: 1px;
	margin-left: -24px;
	padding-left: 20px
	}


ul li        {
	margin-top: 6px;
	margin-bottom: 6px
	}


ul p        {
	margin-top: 6px;
	margin-bottom: 6px
	}


a         {
	color: #00f;
	font-size: 9pt;
	font-family: "Lucida Grande", Arial, sans-serif;
	line-height: 12pt;
	text-decoration: none
	}


a:hover    {
	color: #00f;
	font-family: "Lucida Grande", Arial, sans-serif;
	text-decoration: underline
	}

hr  {
	text-decoration: none;
	border: solid 1px #bfbfbf
	}

td  {
	padding: 6px
	}

#banner     { background-color: #f2f2f2; background-repeat: no-repeat; padding: -2px 6px 0; position: fixed; top: 0; left: 0; width: 100%; height: 1.2em; float: left; border: solid 1px #bfbfbf }

#caticon	         {
margin-top: 3px;
	margin-bottom: -3px;
	margin-right: 5px;
	float: left;
	}

#pagetitle	            {
	margin-top: 12px;
	margin-bottom: 0px;
	margin-left: 40px;
	width: 88%;
	border: solid 1px #fff
	}


#mainbox	         {
	margin-top: 2349px;
	padding-right: 6px
	}

#taskbox  	                     {
	background-color: #e6edff;
	list-style-type: decimal;
	list-style-position: outside;
	margin: 12px 0;
	padding: 2px 12px;
	border: solid 1px #bfbfbf
	}


#taskbox  h2 {
	margin-top: 8;
	margin-bottom: -4px
	}

#machelp   { position: absolute; top: 2px; left: 10px }

#index      { background-color: #f2f2f2; padding-right: 25px; top: 2px; right: 12px; width: auto; float: right }

#next  {
	position: absolute;
	top: 49px;
	left: 88%
	}
	
#asindent {
        margin-left: 22px;
	font-size: 9pt;
	font-family: Verdana, Courier, sans-serif;
        }
        
.bread   {
	color: #00f;
	font-size: 8pt;
	margin: -9px 0 -6px
	}

.leftborder    {
	color: #00f;
	font-size: 8pt;
	margin: -9px 0 -6px;
	padding-top: 2px;
	padding-bottom: 3px;
	padding-left: 8px;
	border-left: 1px solid #bfbfbf
	}

.mult  {
	margin-top: -8px
	}

.blue    {
	background-color: #e6edff;
	margin-top: -3px;
	margin-bottom: -3px;
	padding-top: -3px;
	padding-bottom: -3px
	}

.rightfloater
{
    float: right;
    margin-left: 15px;
}

.rules   {
	border-bottom: 1px dotted #ccc
	}

.dots  {
	border: dotted 1px #ccc
	}

.seealso  {
	margin-top: 4px;
	margin-bottom: 4px
	}
code       {
	color: black;
	font-size: 9pt;
	font-family: Verdana, Courier, sans-serif;
	}
