/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class net_java_sip_communicator_impl_media_protocol_alsa_AlsaStream */

#ifndef _Included_net_java_sip_communicator_impl_media_protocol_alsa_AlsaStream
#define _Included_net_java_sip_communicator_impl_media_protocol_alsa_AlsaStream
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     net_java_sip_communicator_impl_media_protocol_alsa_AlsaStream
 * Method:    jni_alsa_init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_net_java_sip_communicator_impl_media_protocol_alsa_AlsaStream_jni_1alsa_1init
  (JNIEnv *, jobject);

/*
 * Class:     net_java_sip_communicator_impl_media_protocol_alsa_AlsaStream
 * Method:    jni_alsa_read
 * Signature: ([B)V
 */
JNIEXPORT void JNICALL Java_net_java_sip_communicator_impl_media_protocol_alsa_AlsaStream_jni_1alsa_1read
  (JNIEnv *, jobject, jbyteArray);

/*
 * Class:     net_java_sip_communicator_impl_media_protocol_alsa_AlsaStream
 * Method:    jni_alsa_delete
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_net_java_sip_communicator_impl_media_protocol_alsa_AlsaStream_jni_1alsa_1delete
  (JNIEnv *, jobject);

#ifdef __cplusplus
}
#endif
#endif
