/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class net_java_sip_communicator_impl_hid_NativeKeyboard */

#ifndef _Included_net_java_sip_communicator_impl_hid_NativeKeyboard
#define _Included_net_java_sip_communicator_impl_hid_NativeKeyboard
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     net_java_sip_communicator_impl_hid_NativeKeyboard
 * Method:    doKeyAction
 * Signature: (CZ)V
 */
JNIEXPORT void JNICALL Java_net_java_sip_communicator_impl_hid_NativeKeyboard_doKeyAction
  (JNIEnv *, jclass, jchar, jboolean);

/*
 * Class:     net_java_sip_communicator_impl_hid_NativeKeyboard
 * Method:    doSymbolAction
 * Signature: (Ljava/lang/String;Z)V
 */
JNIEXPORT void JNICALL Java_net_java_sip_communicator_impl_hid_NativeKeyboard_doSymbolAction
  (JNIEnv *, jclass, jstring, jboolean);

#ifdef __cplusplus
}
#endif
#endif
