/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class net_java_sip_communicator_impl_sparkle_SparkleActivator */

#ifndef _Included_net_java_sip_communicator_impl_sparkle_SparkleActivator
#define _Included_net_java_sip_communicator_impl_sparkle_SparkleActivator
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     net_java_sip_communicator_impl_sparkle_SparkleActivator
 * Method:    initSparkle
 * Signature: (Ljava/lang/String;ZILjava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_net_java_sip_communicator_impl_sparkle_SparkleActivator_initSparkle
  (JNIEnv *, jclass, jstring, jboolean, jint, jstring, jstring);

#ifdef __cplusplus
}
#endif
#endif
