/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.java.sip.communicator.impl.phonenumbers;

import net.java.sip.communicator.service.protocol.*;
import net.java.sip.communicator.util.*;
import org.osgi.framework.*;

/**
 * Activates PhoneNumberI18nService implementation.
 *
 * @author Damian Minkov
 */
public class PhoneNumberServiceActivator
    implements BundleActivator
{
    /**
     * Our logging.
     */
    private static Logger logger
        = Logger.getLogger(PhoneNumberServiceActivator.class);

    @Override
    public void start(BundleContext bundleContext)
        throws Exception
    {
        bundleContext.registerService(
            PhoneNumberI18nService.class.getName(),
            new PhoneNumberI18nServiceImpl(),
            null);

        if (logger.isInfoEnabled())
            logger.info("Packet Logging Service ...[REGISTERED]");
    }

    @Override
    public void stop(BundleContext bundleContext)
        throws Exception
    {}
}
