/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class net_java_sip_communicator_plugin_addrbook_msoutlook_calendar_CalendarServiceImpl */

#ifndef _Included_net_java_sip_communicator_plugin_addrbook_msoutlook_calendar_CalendarServiceImpl
#define _Included_net_java_sip_communicator_plugin_addrbook_msoutlook_calendar_CalendarServiceImpl
#ifdef __cplusplus
extern "C" {
#endif

/*
 * Class:     net_java_sip_communicator_plugin_addrbook_msoutlook_calendar_CalendarServiceImpl
 * Method:    getAllCalendarItems
 * Signature: (Lnet/java/sip/communicator/plugin/addrbook/msoutlook/calendar/CalendarServiceImpl/PtrOutlookCalendarCallback;)V
 */
JNIEXPORT void JNICALL Java_net_java_sip_communicator_plugin_addrbook_msoutlook_calendar_CalendarServiceImpl_getAllCalendarItems
  (JNIEnv *, jclass, jobject);

/*
 * Class:     net_java_sip_communicator_plugin_addrbook_msoutlook_calendar_CalendarServiceImpl
 * Method:    IMAPIProp_GetProps
 * Signature: (Ljava/lang/String;[JJ)[Ljava/lang/Object;
 */
JNIEXPORT jobjectArray JNICALL Java_net_java_sip_communicator_plugin_addrbook_msoutlook_calendar_CalendarServiceImpl_IMAPIProp_1GetProps
  (JNIEnv *, jclass, jstring, jlongArray, jlong);

#ifdef __cplusplus
}
#endif
#endif
