/*
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.java.sip.communicator.impl.googlecontacts;

/**
 * Response type for OAuth 2 approval code dialog.
 *
 * @author Danny van Heumen
 */
public enum UserResponseType
{
    /**
     * Dialog cancelled.
     */
    CANCELLED,
    /**
     * Dialog confirmed.
     */
    CONFIRMED;
}
