/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.java.sip.communicator.plugin.msofficecomm;

/**
 * Defines the public interface of the out-of-process COM server which
 * integrates Jitsi's presence, IM and VoIP with Microsoft Office/Outlook.
 *
 * @author Lyubomir Marinov
 */
class OutOfProcessServer
{
    static native int start();

    static native int stop();

    /**
     * Prevents the initialization of a new <tt>OutOfProcessServer</tt>
     * instance.
     */
    private OutOfProcessServer()
    {
    }
}
