/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "setup.h"

#ifndef RT_MANIFEST
#define RT_MANIFEST 24
#endif /* #ifndef RT_MANIFEST */
1 RT_MANIFEST "asInvoker.exe.manifest"

/*
 * The .ico may be larger than this .exe with respect to size in bytes. So when
 * building this .exe as a means of distributing a .bspatch, do not include the
 * .ico in the .exe in order to minimize the size in bytes.
 */
#ifndef PACKAGECODE
SCLogoIcon ICON sc-logo.ico
#endif /* #ifndef PACKAGECODE */

STRINGTABLE
BEGIN
    IDS_ISWOW64ACCEPTABLE2 "The version of this file is x86 (32-bit) and the version of Windows you're running is x64 (64-bit). It is recommended to install the x64 (64-bit) version of the program.\012\012Continue anyway?"
    IDS_ISWOW64ACCEPTABLE3 "The version of this file is x86 (32-bit) and the version of Windows you're running is x64 (64-bit). It is recommended to install the x64 (64-bit) version of the program."
    IDS_LASTERRORFORMAT "%1!s! has encountered error %2!d! in file %3!s! at line %4!d!:\012\012%5!s!"
END

#ifdef PACKAGECODE
IDRCDATA_PAYLOAD RCDATA "setup.bspatch"
#else /* #ifdef PACKAGECODE */
IDRCDATA_PAYLOAD RCDATA "setup.msi.xz"
#endif /* #ifdef PACKAGECODE */

1 VERSIONINFO
    FILEVERSION 0,0,0,0
    FILETYPE VFT_APP
    PRODUCTVERSION 0,0,0,0
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904E4"
        BEGIN
            VALUE "Comments", ""
            VALUE "CompanyName", "jitsi.org"
            VALUE "FileDescription", PRODUCTNAME " Setup"
            VALUE "FileVersion", PRODUCTBUILDVERSION
            VALUE "InternalName", "setup"
            VALUE "OriginalFilename", "setup.exe"
            VALUE "ProductName", PRODUCTNAME
            VALUE "ProductVersion", PRODUCTBUILDVERSION
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0409, 1252
    END
END
