/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.java.sip.communicator.service.replacement.directimage;

import net.java.sip.communicator.service.replacement.*;

/**
 * 
 * @author Marin Dzhigarov
 *
 */
public interface DirectImageReplacementService
    extends ReplacementService
{
    /**
     * Returns the size of the image in bytes.
     * @param sourceString the image link.
     * @return the file size in bytes of the image link provided; -1 if the size
     * isn't available or exceeds the max allowed image size.
     */
    public int getImageSize(String sourceString);

    /**
     * Checks if the resource pointed by sourceString is an image.
     * @param sourceString the image link.
     * @return true if the content type of the resource
     * pointed by sourceString is an image.
     */
    public boolean isDirectImage(String sourceString);
}
