/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.java.sip.communicator.impl.ldap;

import net.java.sip.communicator.service.ldap.*;

/**
 * Represents an LDAP search query
 *
 * @author Sebastien Mazy
 */
public class LdapQueryImpl
    implements LdapQuery
{
    /**
     * the query
     * e.g. "John Doe"
     */
    private final String query;

    /**
     * the current state
     */
    private volatile State state;

    /**
     * A simple constructor
     *
     * @param query the query
     */
    public LdapQueryImpl(String query)
    {
        if(query == null)
            throw new RuntimeException("query is null!");
        this.query = query;
        this.state = State.PENDING;
    }

    /**
     * Sets the query state to newState
     *
     * @param newState the query state
     */
    public void setState(State newState)
    {
        this.state = newState;
    }

    /**
     * Returns the query state
     *
     * @return the query state
     */
    public State getState()
    {
        return this.state;
    }

    /**
     * Required by LdapQuery interface
     *
     * Returns the query string
     * e.g "John Doe"
     *
     * @return the query string
     *
     * @see net.java.sip.communicator.service.ldap.LdapQuery#toString
     */
    @Override
    public String toString()
    {
        return this.query;
    }
}
