/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class net_java_sip_communicator_plugin_msofficecomm_OutOfProcessServer */

#ifndef _Included_net_java_sip_communicator_plugin_msofficecomm_OutOfProcessServer
#define _Included_net_java_sip_communicator_plugin_msofficecomm_OutOfProcessServer
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     net_java_sip_communicator_plugin_msofficecomm_OutOfProcessServer
 * Method:    start
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_net_java_sip_communicator_plugin_msofficecomm_OutOfProcessServer_start
  (JNIEnv *, jclass);

/*
 * Class:     net_java_sip_communicator_plugin_msofficecomm_OutOfProcessServer
 * Method:    stop
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_net_java_sip_communicator_plugin_msofficecomm_OutOfProcessServer_stop
  (JNIEnv *, jclass);

#ifdef __cplusplus
}
#endif
#endif
