/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class net_java_sip_communicator_util_launchutils_AEGetURLEventHandler */

#ifndef _Included_net_java_sip_communicator_util_launchutils_AEGetURLEventHandler
#define _Included_net_java_sip_communicator_util_launchutils_AEGetURLEventHandler
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     net_java_sip_communicator_util_launchutils_AEGetURLEventHandler
 * Method:    setAEGetURLListener
 * Signature: (Lnet/java/sip/communicator/util/launchutils/AEGetURLEventHandler/IAEGetURLListener;)V
 */
JNIEXPORT void JNICALL Java_net_java_sip_communicator_util_launchutils_AEGetURLEventHandler_setAEGetURLListener
  (JNIEnv *, jclass, jobject);

#ifdef __cplusplus
}
#endif
#endif
/* Header for class net_java_sip_communicator_util_launchutils_AEGetURLEventHandler_IAEGetURLListener */

#ifndef _Included_net_java_sip_communicator_util_launchutils_AEGetURLEventHandler_IAEGetURLListener
#define _Included_net_java_sip_communicator_util_launchutils_AEGetURLEventHandler_IAEGetURLListener
#ifdef __cplusplus
extern "C" {
#endif
#ifdef __cplusplus
}
#endif
#endif
