/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class net_java_sip_communicator_impl_netaddr_HardwareAddressRetriever */

#ifndef _Included_net_java_sip_communicator_impl_netaddr_HardwareAddressRetriever
#define _Included_net_java_sip_communicator_impl_netaddr_HardwareAddressRetriever
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     net_java_sip_communicator_impl_netaddr_HardwareAddressRetriever
 * Method:    getHardwareAddress
 * Signature: (Ljava/lang/String;)[B
 */
JNIEXPORT jbyteArray JNICALL Java_net_java_sip_communicator_impl_netaddr_HardwareAddressRetriever_getHardwareAddress
  (JNIEnv *, jclass, jstring);

#ifdef __cplusplus
}
#endif
#endif
