/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.java.sip.communicator.service.replacement;

/**
 * Property for Replacement Service
 *
 * @author Purvesh Sahoo
 */
public final class ReplacementProperty
{
    /**
     * The replacement proposal property.
     */
    public static final String REPLACEMENT_PROPOSAL =
        "plugin.chatconfig.replacement.proposal.enable";

    /**
     *  The replacement property.
     */
    public static final String REPLACEMENT_ENABLE =
        "net.java.sip.communicator.service.replacement.enable";

    /**
     * Returns the property name of individual replacement sources
     *
     * @param source the replacement source name.
     * @return the property name of the specified source as will be stored in
     *         the properties file.
     */
    public static String getPropertyName(String source)
    {
        return "net.java.sip.communicator.service.replacement."
                + source
                + ".enable";
    }
}
